@@ -7,41 +7,42 @@ use vars qw($VERSION @EXPORT @EXPORT_OK);
 use base 'Exporter';
 @EXPORT_OK = @EXPORT = qw( hash2xml xml2hash );
 
-$VERSION = '0.32';
+$VERSION = '0.33';
 
 require XSLoader;
 XSLoader::load('XML::Hash::XS', $VERSION);
 
 use vars qw($method $output $root $version $encoding $utf8 $indent $canonical
     $use_attr $content $xml_decl $doc $max_depth $attr $text $trim $cdata
-    $comm $buf_size $keep_root
+    $comm $buf_size $keep_root $force_array
 );
 
 # 'NATIVE' or 'LX'
-our $method    = 'NATIVE';
+$method      = 'NATIVE';
 
 # native options
-$output     = undef;
-$root       = 'root';
-$version    = '1.0';
-$encoding   = '';
-$utf8       = 1;
-$indent     = 0;
-$canonical  = 0;
-$use_attr   = 0;
-$content    = undef;
-$xml_decl   = 1;
-$keep_root  = 0;#
-$doc        = 0;
-$max_depth  = 1024;
-$buf_size   = 4096;
-$trim       = 0;
+$output      = undef;
+$root        = 'root';
+$version     = '1.0';
+$encoding    = '';
+$utf8        = 1;
+$indent      = 0;
+$canonical   = 0;
+$use_attr    = 0;
+$content     = undef;
+$xml_decl    = 1;
+$keep_root   = 0;#
+$doc         = 0;
+$max_depth   = 1024;
+$buf_size    = 4096;
+$trim        = 0;
+$force_array = undef;
 
 # XML::Hash::LX options
-$attr       = '-';
-$text       = '#text';
-$cdata      = undef;
-$comm       = undef;
+$attr        = '-';
+$text        = '#text';
+$cdata       = undef;
+$comm        = undef;
 
 1;
 __END__
@@ -213,10 +214,14 @@ if use_attr is "1", converter will be use the attributes.
 
 if use_attr is "0", converter will be use tags only.
 
-=item content [ = undef ] I<# hash2xml>
+=item content [ = undef ] I<# hash2xml+xml2hash>
 
 if defined that the key name for the text content(used only if use_attr=1).
 
+=item force_array => [ = undef ] I<# xml2hash>
+
+This option is similar to "ForceArray" from XMl::Simple module: L<https://metacpan.org/pod/XML::Simple#ForceArray-1-in---important>.
+
 =item xml_decl [ = 1 ] I<# hash2xml>
 
 if xml_decl is "1", output will start with the XML declaration '<?xml version="1.0" encoding="utf-8"?>'.
